from libc.stdint cimport int8_t, int16_t, int32_t, int64_t, intptr_t
from libc.stdint cimport uint8_t, uint16_t, uint32_t, uint64_t, uintptr_t
cdef extern from *:
  ctypedef bint bool
  ctypedef struct va_list

cdef extern from *:

  ctypedef struct Inner_1:
    uint8_t bytes[1];

  ctypedef struct Outer_1:
    Inner_1 inner;

  ctypedef struct Inner_2:
    uint8_t bytes[2];

  ctypedef struct Outer_2:
    Inner_2 inner;

  Outer_1 one();

  Outer_2 two();



from libc.stdint cimport int8_t, int16_t, int32_t, int64_t, intptr_t
from libc.stdint cimport uint8_t, uint16_t, uint32_t, uint64_t, uintptr_t
cdef extern from *:
  ctypedef bint bool
  ctypedef struct va_list

cdef extern from *:

  const uintptr_t TITLE_SIZE # = 80

  ctypedef int8_t CArrayString_TITLE_SIZE[TITLE_SIZE];

  ctypedef int8_t CArrayString_40[40];

  cdef struct Book:
    CArrayString_TITLE_SIZE title;
    CArrayString_40 author;

  void root(Book *a);



#if 0
DEF DEFINED = 1
DEF NOT_DEFINED = 0
#endif


from libc.stdint cimport int8_t, int16_t, int32_t, int64_t, intptr_t
from libc.stdint cimport uint8_t, uint16_t, uint32_t, uint64_t, uintptr_t
cdef extern from *:
  ctypedef bint bool
  ctypedef struct va_list

cdef extern from *:

  IF NOT_DEFINED:
    const int32_t DEFAULT_X # = 8

  IF DEFINED:
    const int32_t DEFAULT_X # = 42

  IF (NOT_DEFINED or DEFINED):
    ctypedef struct Foo:
      int32_t x;

  IF NOT_DEFINED:
    ctypedef struct Bar:
      Foo y;

  IF DEFINED:
    ctypedef struct Bar:
      Foo z;

  ctypedef struct Root:
    Bar w;

  void root(Root a);



from libc.stdint cimport int8_t, int16_t, int32_t, int64_t, intptr_t
from libc.stdint cimport uint8_t, uint16_t, uint32_t, uint64_t, uintptr_t
cdef extern from *:
  ctypedef bint bool
  ctypedef struct va_list

cdef extern from *:

  ctypedef void (*A)();

  ctypedef void (*B)();

  ctypedef bool (*C)(int32_t, int32_t);

  ctypedef bool (*(*D)(int32_t))(float);

  ctypedef const int32_t (*(*E)())[16];

  ctypedef const int32_t *F;

  ctypedef const int32_t *const *G;

  ctypedef int32_t *const *H;

  ctypedef const int32_t (*I)[16];

  ctypedef double (**J)(float);

  ctypedef int32_t K[16];

  ctypedef const int32_t *L[16];

  ctypedef bool (*M[16])(int32_t, int32_t);

  ctypedef void (*N[16])(int32_t, int32_t);

  ctypedef void (*P)(int32_t named1st, bool, bool named3rd, int32_t _);

  void (*O())();

  void root(A a, B b, C c, D d, E e, F f, G g, H h, I i, J j, K k, L l, M m, N n, P p);



from libc.stdint cimport int8_t, int16_t, int32_t, int64_t, intptr_t
from libc.stdint cimport uint8_t, uint16_t, uint32_t, uint64_t, uintptr_t
cdef extern from *:
  ctypedef bint bool
  ctypedef struct va_list

cdef extern from *:

  ctypedef struct StylePoint_i32:
    int32_t x;
    int32_t y;

  ctypedef struct StylePoint_f32:
    float x;
    float y;

  cdef enum:
    Foo_i32,
    Bar_i32,
    Baz_i32,
    Bazz_i32,
  ctypedef uint8_t StyleFoo_i32_Tag;

  ctypedef struct StyleFoo_Body_i32:
    StyleFoo_i32_Tag tag;
    int32_t x;
    StylePoint_i32 y;
    StylePoint_f32 z;

  ctypedef union StyleFoo_i32:
    StyleFoo_i32_Tag tag;
    StyleFoo_Body_i32 foo;
    int32_t bar;
    StylePoint_i32 baz;

  ctypedef enum StyleBar_i32_Tag:
    Bar1_i32,
    Bar2_i32,
    Bar3_i32,
    Bar4_i32,

  ctypedef struct StyleBar1_Body_i32:
    int32_t x;
    StylePoint_i32 y;
    StylePoint_f32 z;
    int32_t (*u)(int32_t);

  ctypedef struct StyleBar_i32:
    StyleBar_i32_Tag tag;
    StyleBar1_Body_i32 bar1;
    int32_t bar2;
    StylePoint_i32 bar3;

  ctypedef struct StylePoint_u32:
    uint32_t x;
    uint32_t y;

  ctypedef enum StyleBar_u32_Tag:
    Bar1_u32,
    Bar2_u32,
    Bar3_u32,
    Bar4_u32,

  ctypedef struct StyleBar1_Body_u32:
    int32_t x;
    StylePoint_u32 y;
    StylePoint_f32 z;
    int32_t (*u)(int32_t);

  ctypedef struct StyleBar_u32:
    StyleBar_u32_Tag tag;
    StyleBar1_Body_u32 bar1;
    uint32_t bar2;
    StylePoint_u32 bar3;

  cdef enum:
    Baz1,
    Baz2,
    Baz3,
  ctypedef uint8_t StyleBaz_Tag;

  ctypedef union StyleBaz:
    StyleBaz_Tag tag;
    StyleBar_u32 baz1;
    StylePoint_i32 baz2;

  cdef enum:
    Taz1,
    Taz2,
    Taz3,
  ctypedef uint8_t StyleTaz_Tag;

  ctypedef struct StyleTaz:
    StyleTaz_Tag tag;
    StyleBar_u32 taz1;
    StyleBaz taz2;

  void foo(const StyleFoo_i32 *foo,
           const StyleBar_i32 *bar,
           const StyleBaz *baz,
           const StyleTaz *taz);



from libc.stdint cimport int8_t, int16_t, int32_t, int64_t, intptr_t
from libc.stdint cimport uint8_t, uint16_t, uint32_t, uint64_t, uintptr_t
cdef extern from *:
  ctypedef bint bool
  ctypedef struct va_list

cdef extern from *:

  const int32_t C_H # = 10

  cdef enum:
    x # = 0,
    y # = 1,
  ctypedef uint8_t C_E;

  ctypedef struct C_A:
    pass

  ctypedef struct C_C:
    pass

  ctypedef struct C_AwesomeB:
    int32_t x;
    float y;

  ctypedef union C_D:
    int32_t x;
    float y;

  ctypedef C_A C_F;

  const intptr_t C_I # = <intptr_t><C_F*>10

  extern const int32_t G;

  void root(const C_A *a, C_AwesomeB b, C_C c, C_D d, C_E e, C_F f);



from libc.stdint cimport int8_t, int16_t, int32_t, int64_t, intptr_t
from libc.stdint cimport uint8_t, uint16_t, uint32_t, uint64_t, uintptr_t
cdef extern from *:
  ctypedef bint bool
  ctypedef struct va_list

cdef extern from *:

  cdef enum:
    X # = 2,
    Y,
  ctypedef uint32_t C;

  ctypedef struct A:
    int32_t m0;

  ctypedef struct B:
    int32_t x;
    float y;

  cdef enum:
    Foo,
    Bar,
    Baz,
  ctypedef uint8_t F_Tag;

  ctypedef struct Bar_Body:
    F_Tag tag;
    uint8_t x;
    int16_t y;

  ctypedef union F:
    F_Tag tag;
    int16_t foo;
    Bar_Body bar;

  cdef enum:
    Hello,
    There,
    Everyone,
  ctypedef uint8_t H_Tag;

  ctypedef struct There_Body:
    uint8_t x;
    int16_t y;

  ctypedef struct H:
    H_Tag tag;
    int16_t hello;
    There_Body there;

  void root(A x, B y, C z, F f, H h);



#define CBINDGEN_PACKED     __attribute__ ((packed))
#define CBINDGEN_ALIGNED(n) __attribute__ ((aligned(n)))


from libc.stdint cimport int8_t, int16_t, int32_t, int64_t, intptr_t
from libc.stdint cimport uint8_t, uint16_t, uint32_t, uint64_t, uintptr_t
cdef extern from *:
  ctypedef bint bool
  ctypedef struct va_list

cdef extern from *:

  ctypedef struct RustAlign4Struct:
    pass

  ctypedef struct RustAlign4Union:
    pass

  ctypedef struct RustPackedStruct:
    pass

  ctypedef struct RustPackedUnion:
    pass

  ctypedef struct UnsupportedAlign4Enum:
    pass

  ctypedef struct UnsupportedPacked4Struct:
    pass

  ctypedef struct UnsupportedPacked4Union:
    pass

  ctypedef struct Align1Struct:
    uintptr_t arg1;
    uint8_t *arg2;

  ctypedef struct Align2Struct:
    uintptr_t arg1;
    uint8_t *arg2;

  ctypedef struct Align4Struct:
    uintptr_t arg1;
    uint8_t *arg2;

  ctypedef struct Align8Struct:
    uintptr_t arg1;
    uint8_t *arg2;

  ctypedef struct Align32Struct:
    uintptr_t arg1;
    uint8_t *arg2;

  ctypedef packed struct PackedStruct:
    uintptr_t arg1;
    uint8_t *arg2;

  ctypedef union Align1Union:
    uintptr_t variant1;
    uint8_t *variant2;

  ctypedef union Align4Union:
    uintptr_t variant1;
    uint8_t *variant2;

  ctypedef union Align16Union:
    uintptr_t variant1;
    uint8_t *variant2;

  ctypedef union PackedUnion:
    uintptr_t variant1;
    uint8_t *variant2;



#ifdef __clang__
#define CBINDGEN_NONNULL _Nonnull
#else
#define CBINDGEN_NONNULL
#endif


from libc.stdint cimport int8_t, int16_t, int32_t, int64_t, intptr_t
from libc.stdint cimport uint8_t, uint16_t, uint32_t, uint64_t, uintptr_t
cdef extern from *:
  ctypedef bint bool
  ctypedef struct va_list

cdef extern from *:

  cdef struct Opaque:
    pass

  cdef struct References:
    const Opaque *a;
    Opaque *b;
    const Opaque *c;
    Opaque *d;

  cdef struct Pointers_u64:
    float *a;
    uint64_t *b;
    Opaque *c;
    uint64_t **d;
    float **e;
    Opaque **f;
    uint64_t *g;
    int32_t *h;
    int32_t **i;
    const uint64_t *j;
    uint64_t *k;

  void value_arg(References arg);

  void mutltiple_args(int32_t *arg, Pointers_u64 *foo, Opaque **d);

  void ref_arg(const Pointers_u64 *arg);

  void mut_ref_arg(Pointers_u64 *arg);

  void optional_ref_arg(const Pointers_u64 *arg);

  void optional_mut_ref_arg(Pointers_u64 *arg);

  void nullable_const_ptr(const Pointers_u64 *arg);

  void nullable_mut_ptr(Pointers_u64 *arg);



from libc.stdint cimport int8_t, int16_t, int32_t, int64_t, intptr_t
from libc.stdint cimport uint8_t, uint16_t, uint32_t, uint64_t, uintptr_t
cdef extern from *:
  ctypedef bint bool
  ctypedef struct va_list

cdef extern from *:

  const uint8_t EXPORT_ME_TOO # = 42

  ctypedef struct ExportMe:
    uint64_t val;

  ctypedef struct ExportMe2:
    uint64_t val;

  void export_me(ExportMe *val);

  void export_me_2(ExportMe2*);

  void from_really_nested_mod();



from libc.stdint cimport int8_t, int16_t, int32_t, int64_t, intptr_t
from libc.stdint cimport uint8_t, uint16_t, uint32_t, uint64_t, uintptr_t
cdef extern from *:
  ctypedef bint bool
  ctypedef struct va_list

cdef extern from *:

  ctypedef struct Parser_40__41:
    uint8_t *buf;
    uintptr_t len;

  ctypedef struct Parser_123__125:
    uint8_t *buf;
    uintptr_t len;

  void init_parens_parser(Parser_40__41 *p, uint8_t *buf, uintptr_t len);

  void destroy_parens_parser(Parser_40__41 *p);

  void init_braces_parser(Parser_123__125 *p, uint8_t *buf, uintptr_t len);



#if 0
''' '
#endif

#ifdef __cplusplus
template <typename T>
using Box = T*;
#endif

#if 0
' '''
#endif


from libc.stdint cimport int8_t, int16_t, int32_t, int64_t, intptr_t
from libc.stdint cimport uint8_t, uint16_t, uint32_t, uint64_t, uintptr_t
cdef extern from *:
  ctypedef bint bool
  ctypedef struct va_list

cdef extern from *:

  cdef enum:
    a1 # = 0,
    a2 # = 2,
    a3,
    a4 # = 5,
  ctypedef uint64_t A;

  cdef enum:
    b1 # = 0,
    b2 # = 2,
    b3,
    b4 # = 5,
  ctypedef uint32_t B;

  cdef enum:
    c1 # = 0,
    c2 # = 2,
    c3,
    c4 # = 5,
  ctypedef uint16_t C;

  cdef enum:
    d1 # = 0,
    d2 # = 2,
    d3,
    d4 # = 5,
  ctypedef uint8_t D;

  cdef enum:
    e1 # = 0,
    e2 # = 2,
    e3,
    e4 # = 5,
  ctypedef uintptr_t E;

  cdef enum:
    f1 # = 0,
    f2 # = 2,
    f3,
    f4 # = 5,
  ctypedef intptr_t F;

  cdef enum L:
    l1,
    l2,
    l3,
    l4,

  cdef enum:
    m1 # = -1,
    m2 # = 0,
    m3 # = 1,
  ctypedef int8_t M;

  cdef enum N:
    n1,
    n2,
    n3,
    n4,

  cdef enum:
    o1,
    o2,
    o3,
    o4,
  ctypedef int8_t O;

  cdef struct J:
    pass

  cdef struct K:
    pass

  cdef struct Opaque:
    pass

  cdef enum:
    Foo,
    Bar,
    Baz,
  ctypedef uint8_t G_Tag;

  cdef struct Bar_Body:
    G_Tag tag;
    uint8_t x;
    int16_t y;

  cdef union G:
    G_Tag tag;
    int16_t foo;
    Bar_Body bar;

  cdef enum H_Tag:
    H_Foo,
    H_Bar,
    H_Baz,

  cdef struct H_Bar_Body:
    uint8_t x;
    int16_t y;

  cdef struct H:
    H_Tag tag;
    int16_t foo;
    H_Bar_Body bar;

  cdef enum:
    ExI_Foo,
    ExI_Bar,
    ExI_Baz,
  ctypedef uint8_t ExI_Tag;

  cdef struct ExI_Bar_Body:
    uint8_t x;
    int16_t y;

  cdef struct ExI:
    ExI_Tag tag;
    int16_t foo;
    ExI_Bar_Body bar;

  cdef enum:
    P0,
    P1,
  ctypedef uint8_t P_Tag;

  cdef struct P1_Body:
    uint8_t _0;
    uint8_t _1;
    uint8_t _2;

  cdef struct P:
    P_Tag tag;
    uint8_t p0;
    P1_Body p1;

  cdef enum Q_Tag:
    Ok,
    Err,

  cdef struct Q:
    Q_Tag tag;
    uint32_t *ok;
    uint32_t err;

  cdef enum R_Tag:
    IRFoo,
    IRBar,
    IRBaz,

  cdef struct IRBar_Body:
    uint8_t x;
    int16_t y;

  cdef struct R:
    R_Tag tag;
    int16_t IRFoo;
    IRBar_Body IRBar;

  void root(Opaque *opaque,
            A a,
            B b,
            C c,
            D d,
            E e,
            F f,
            G g,
            H h,
            ExI i,
            J j,
            K k,
            L l,
            M m,
            N n,
            O o,
            P p,
            Q q,
            R r);

#if 0
''' '
#endif

#include <stddef.h>
#include "testing-helpers.h"
static_assert(offsetof(CBINDGEN_STRUCT(P), tag) == 0, "unexpected offset for tag");
static_assert(offsetof(CBINDGEN_STRUCT(P), p0) == 1, "unexpected offset for p0");
static_assert(offsetof(CBINDGEN_STRUCT(P), p0) == 1, "unexpected offset for p1");
static_assert(sizeof(CBINDGEN_STRUCT(P)) == 4, "unexpected size for P");

#if 0
' '''
#endif



#define DEPRECATED_FUNC __attribute__((deprecated))
#define DEPRECATED_STRUCT __attribute__((deprecated))
#define DEPRECATED_ENUM __attribute__((deprecated))
#define DEPRECATED_FUNC_WITH_NOTE(...) __attribute__((deprecated(__VA_ARGS__)))
#define DEPRECATED_STRUCT_WITH_NOTE(...) __attribute__((deprecated(__VA_ARGS__)))
#define DEPRECATED_ENUM_WITH_NOTE(...) __attribute__((deprecated(__VA_ARGS__)))


from libc.stdint cimport int8_t, int16_t, int32_t, int64_t, intptr_t
from libc.stdint cimport uint8_t, uint16_t, uint32_t, uint64_t, uintptr_t
cdef extern from *:
  ctypedef bint bool
  ctypedef struct va_list

cdef extern from *:

  cdef enum:
    A # = 0,
  ctypedef int32_t DeprecatedEnum;

  cdef enum:
    B # = 0,
  ctypedef int32_t DeprecatedEnumWithNote;

  ctypedef struct DeprecatedStruct:
    int32_t a;

  ctypedef struct DeprecatedStructWithNote:
    int32_t a;

  void deprecated_without_note();

  void deprecated_without_bracket();

  void deprecated_with_note();

  void deprecated_with_note_and_since();

  void deprecated_with_note_which_requires_to_be_escaped();

  void dummy(DeprecatedEnum a,
             DeprecatedEnumWithNote b,
             DeprecatedStruct c,
             DeprecatedStructWithNote d);



from libc.stdint cimport int8_t, int16_t, int32_t, int64_t, intptr_t
from libc.stdint cimport uint8_t, uint16_t, uint32_t, uint64_t, uintptr_t
cdef extern from *:
  ctypedef bint bool
  ctypedef struct va_list

cdef extern from *:

  # Constants shared by multiple CSS Box Alignment properties
  #
  # These constants match Gecko's `NS_STYLE_ALIGN_*` constants.
  ctypedef struct AlignFlags:
    uint8_t bits;
  # 'auto'
  const AlignFlags AlignFlags_AUTO # = <AlignFlags>{ <uint8_t>0 }
  # 'normal'
  const AlignFlags AlignFlags_NORMAL # = <AlignFlags>{ <uint8_t>1 }
  # 'start'
  const AlignFlags AlignFlags_START # = <AlignFlags>{ <uint8_t>(1 << 1) }
  # 'end'
  const AlignFlags AlignFlags_END # = <AlignFlags>{ <uint8_t>(1 << 2) }
  const AlignFlags AlignFlags_ALIAS # = <AlignFlags>{ <uint8_t>(AlignFlags_END).bits }
  # 'flex-start'
  const AlignFlags AlignFlags_FLEX_START # = <AlignFlags>{ <uint8_t>(1 << 3) }
  const AlignFlags AlignFlags_MIXED # = <AlignFlags>{ <uint8_t>(((1 << 4) | (AlignFlags_FLEX_START).bits) | (AlignFlags_END).bits) }
  const AlignFlags AlignFlags_MIXED_SELF # = <AlignFlags>{ <uint8_t>(((1 << 5) | (AlignFlags_FLEX_START).bits) | (AlignFlags_END).bits) }

  ctypedef struct DebugFlags:
    uint32_t bits;
  # Flag with the topmost bit set of the u32
  const DebugFlags DebugFlags_BIGGEST_ALLOWED # = <DebugFlags>{ <uint32_t>(1 << 31) }

  ctypedef struct LargeFlags:
    uint64_t bits;
  # Flag with a very large shift that usually would be narrowed.
  const LargeFlags LargeFlags_LARGE_SHIFT # = <LargeFlags>{ <uint64_t>(1ull << 44) }
  const LargeFlags LargeFlags_INVERTED # = <LargeFlags>{ <uint64_t>~(LargeFlags_LARGE_SHIFT).bits }

  ctypedef struct OutOfLine:
    uint32_t _0;
  const OutOfLine OutOfLine_A # = <OutOfLine>{ <uint32_t>1 }
  const OutOfLine OutOfLine_B # = <OutOfLine>{ <uint32_t>2 }
  const OutOfLine OutOfLine_AB # = <OutOfLine>{ <uint32_t>((OutOfLine_A)._0 | (OutOfLine_B)._0) }

  void root(AlignFlags flags,
            DebugFlags bigger_flags,
            LargeFlags largest_flags,
            OutOfLine out_of_line);



from libc.stdint cimport int8_t, int16_t, int32_t, int64_t, intptr_t
from libc.stdint cimport uint8_t, uint16_t, uint32_t, uint64_t, uintptr_t
cdef extern from *:
  ctypedef bint bool
  ctypedef struct va_list

cdef extern from *:

  ctypedef struct Fns:
    void (*noArgs)();
    void (*anonymousArg)(int32_t);
    int32_t (*returnsNumber)();
    int8_t (*namedArgs)(int32_t first, int16_t snd);
    int8_t (*namedArgsWildcards)(int32_t _, int16_t named, int64_t _1);

  void root(Fns _fns);

  void no_return();



from libc.stdint cimport int8_t, int16_t, int32_t, int64_t, intptr_t
from libc.stdint cimport uint8_t, uint16_t, uint32_t, uint64_t, uintptr_t
cdef extern from *:
  ctypedef bint bool
  ctypedef struct va_list

cdef extern from *:

  ctypedef void (*MyCallback)(uintptr_t a, uintptr_t b);

  ctypedef void (*MyOtherCallback)(uintptr_t a,
                                   uintptr_t lot,
                                   uintptr_t of,
                                   uintptr_t args,
                                   uintptr_t and_then_some);

  void my_function(MyCallback a, MyOtherCallback b);



from libc.stdint cimport int8_t, int16_t, int32_t, int64_t, intptr_t
from libc.stdint cimport uint8_t, uint16_t, uint32_t, uint64_t, uintptr_t
cdef extern from *:
  ctypedef bint bool
  ctypedef struct va_list

cdef extern from *:

  ctypedef const char *Str;

  ctypedef struct HashTable_Str__c_char__false:
    uintptr_t num_buckets;
    uintptr_t capacity;
    uint8_t *occupied;
    Str *keys;
    char *vals;

  ctypedef HashTable_Str__c_char__false MySet;

  ctypedef void (*SetCallback)(Str key);

  ctypedef struct HashTable_Str__u64__true:
    uintptr_t num_buckets;
    uintptr_t capacity;
    uint8_t *occupied;
    Str *keys;
    uint64_t *vals;

  ctypedef void (*MapCallback)(Str key, uint64_t val);

  MySet *new_set();

  void set_for_each(const MySet *set, SetCallback callback);

  HashTable_Str__u64__true *new_map();

  void map_for_each(const HashTable_Str__u64__true *map, MapCallback callback);



#define CBINDGEN_PACKED        __attribute__ ((packed))
#define CBINDGEN_ALIGNED(n)    __attribute__ ((aligned(n)))


from libc.stdint cimport int8_t, int16_t, int32_t, int64_t, intptr_t
from libc.stdint cimport uint8_t, uint16_t, uint32_t, uint64_t, uintptr_t
cdef extern from *:
  ctypedef bint bool
  ctypedef struct va_list

cdef extern from *:

  cdef struct OpaqueAlign16Union:
    pass

  cdef struct OpaqueAlign1Struct:
    pass

  cdef struct OpaqueAlign1Union:
    pass

  cdef struct OpaqueAlign2Struct:
    pass

  cdef struct OpaqueAlign32Struct:
    pass

  cdef struct OpaqueAlign4Struct:
    pass

  cdef struct OpaqueAlign4Union:
    pass

  cdef struct OpaqueAlign8Struct:
    pass

  cdef packed struct PackedStruct:
    uintptr_t arg1;
    uint8_t *arg2;

  cdef union PackedUnion:
    uintptr_t variant1;
    uint8_t *variant2;



from libc.stdint cimport int8_t, int16_t, int32_t, int64_t, intptr_t
from libc.stdint cimport uint8_t, uint16_t, uint32_t, uint64_t, uintptr_t
cdef extern from *:
  ctypedef bint bool
  ctypedef struct va_list

cdef extern from *:

  ctypedef struct ArrayVec_____u8__100:
    uint8_t *xs[100];
    uint32_t len;

  int32_t push(ArrayVec_____u8__100 *v, uint8_t *elem);
